%% reference: Xuchun Li, Lei Wang, Eric Sung - "A Study of AdaBoost with SVM Based Weak Learners" - IJCNN 2005

function [Accuracy Specificity Sensitivity alpha weakres] = Diverse_AdaBoost_LVQ ()
   %cd /media/xcraft/Documents/phd/medical_images/my_papers/IWANN09/src

   load /home/maite/Documentos/IWANN_IWINAC_IDEAL2009/codigo_IWANN09/spm_segmented_oasis_females.mat

   [C P] = extractVoxelIntensitiesWithinClusters (female_crystal_brain_cov0, CONTROLES, PACIENTES);
%   [C P] = extractMeanAndStdDevFromEachCluster   (female_crystal_brain_cov0, CONTROLES, PACIENTES);

   [L, num_clusters] = bwlabeln (female_crystal_brain_cov0);

   k_de_kfold = 10;
   p = cvpartition(size(C,1),'kfold',k_de_kfold);
   c = cvpartition(size(C,1),'kfold',k_de_kfold);
   
   control_label=2;
   patient_label=1;

   learn_rate_min  = 0.001;
   learn_rate_max  = 0.8;
   learn_rate_step = 0.05;
   min_diversity   = 0.5;
   
 %%Parametros por defecto de las LVQ
   epochs         = 150;
   goal           = 0.01;
   show           = Inf;
   showWindow     = false;
   hidden_neurons = 5;
  
   for k=1:k_de_kfold;

      learning_rate    = learn_rate_max;
      i                = 1;
      total_diversity  =  0;
   
      %compute folds
      %training set used to compute the weights for each clusterSVM (our weaklearners)
      C_training = C(training (c,k) == 1,:,:);
      P_training = P(training (p,k) == 1,:,:);
   
      %test set used to validate the adaboost
      C_test     = C(training (c,k) == 0,:,:);
      P_test     = P(training (p,k) == 0,:,:);

      training_patients_no = size(P_training,1);
      training_controls_no = size(C_training,1);
      training_subjects_no = training_patients_no + training_controls_no;
      sample_weights       = ones(1,training_subjects_no)'/training_subjects_no;

      %initialize training set
         [training_data, training_label, feat_no] = alignAndLabelDataForNN (C_training, P_training, control_label, patient_label);
         label_ind=ind2vec(training_label);

      %weight training
      while (learning_rate > learn_rate_min);
          
          %creating neural network and training with training set
          numTrain = training_subjects_no;
          priori_1 = training_controls_no/numTrain;
          priori_2 = training_patients_no/numTrain;

          lvqnet = newlvq(minmax(training_data'), hidden_neurons, [priori_1 priori_2], learning_rate);
          lvqnet.trainParam.epochs     = epochs;
          lvqnet.trainParam.goal       = goal;
          lvqnet.trainParam.show       = show;
          lvqnet.trainParam.showWindow = showWindow;

          [trained_net, train_record] = train(lvqnet, training_data', label_ind);

         %classification test with training data
         predict_label = vec2ind ( sim (trained_net, training_data') );

         %adaboost compute training errors
         training_error = 0;
         diversity      = 0;
         for s=1:size(training_data,1);
            if (predict_label(s) ~= training_label(s));
               training_error = training_error  + sample_weights(s);
               diversity      = diversity       + 1;
            end;
         end;

         current_diversity = total_diversity + ( diversity/(i*training_subjects_no) )
         if ( (training_error > 0.5) && ( current_diversity > min_diversity ) )
            learning_rate = learning_rate - learn_rate_step;
         else
            %save the weaklearner, save the world!
            weaklearners(i).net = trained_net;

            training_errors (i) = training_error;
            diversities     (i) = diversity;

            %compute total_diversity
            total_diversity = current_diversity;

            %adaboost set classifier weights
            alpha(i) = 1;
            if (training_errors(i) ~= 0);
               alpha (i) = 0.5 * log((1-training_errors(i)) / training_errors(i));
            end;
            
            sample_weights = sample_weights .* exp( -alpha(i) .* training_label .* predict_label');
            sample_weights = sample_weights/sum(sample_weights);

            %update sigma and loop counter for the next iteration
            learning_rate = learning_rate - learn_rate_step;
            i = i + 1;
         end;
      end;

      %classify test set
      [test_data, test_label, feat_no] = alignAndLabelDataForNN (C_test, P_test, control_label, patient_label);
      label_ind=ind2vec(test_label);

      for i=1:size(alpha,1);
         boosted_results(k,i).output = vec2ind ( sim (weaklearners(i).net, test_data') );
         [weakres(k,i).TP weakres(k,i).TN weakres(k,i).FP weakres(k,i).FN weakres(k,i).Accuracy weakres(k,i).Specificity weakres(k,i).Sensitivity] = computeErrors (boosted_results(k,i).target', boosted_results(k,i).output);
      end;
      
      %classify and evaluate adaboost
      total_tests = 0;
      TP = 0;
      TN = 0;
      FP = 0;
      FN = 0;
      
      %Transformamos los labels controles=1 y pacientes=-1 para
      %calcular la clasificacion el cjto de weaklearners
      trans_boosted_results.output=zeros(size(boosted_results(k,1).output));
      trans_boosted_results.target=zeros(size(boosted_results(k,1).target));
      
      trans_boosted_results.output(find(boosted_results(k,:).output==control_label)) = 1;
      trans_boosted_results.output(find(boosted_results(k,:).output==patient_label)) =-1;
      
      trans_boosted_results.target(find(boosted_results(k,:).target==control_label)) = 1;
      trans_boosted_results.target(find(boosted_results(k,:).target==patient_label)) =-1;

      for s=1:size(test_data,1);

         %adaboost classification with alpha (weaklearner weights)
         %if negative: class = -1
         %else if positive: class = 1
         %else if zero: only the first cluster SVM prediction with maximum weight will be taken into account
         class = 0;
         for c=1:size(alpha,1);
            class = class + alpha(c) * trans_boosted_results (c).output(s);
         end;
         if (class == 0);
            [max_value max_idx] = max(mean_alpha);
            class = trans_boosted_results (max_idx(1)).output(s);
         end;

         subj_predictions(s) = sign(class)

         %counting and compare with target label
         target_label = test_label(s)

         if (test_label(s) == -1); %positive
            if   (subj_predictions(s) == -1); TP = TP + 1;
            else                              FN = FN + 1;
            end;
         else %negative
            if   (subj_predictions(s) ==  1); TN = TN + 1;
            else                              FP = FP + 1;
            end;
         end;

         total_tests = total_tests + 1;
      end;

      clear weaklearners subj_predictions
   end;

   alpha

   Accuracy    = (TP + TN) / total_tests
   Specificity = TN / (TN+FP)
   Sensitivity = TP / (TP+FN)


