function [best_gamma best_C] = estimateBestRBFParameters (data, label)

 %k-fold cross_validation repetition number
 times = 5;

 %number of folds
 folds = 2;
 p = cvpartition(size(data,1), 'kfold', folds);

 min_gamma_exp = -3;
 max_gamma_exp =  0;
 stp_gamma_exp =  0.1;
 gamma_base    = 10;

     min_C_exp = -3;
     max_C_exp =  2;
     stp_C_exp =  0.1;
 C_base        = 10;

 best_brier_score = Inf;

 for i=1:times;
    for k=1:folds;
        %separate data in training set and test set
        train_set   = data (training (p,k) == 1,:,:);
        train_label = label(training (p,k) == 1,:,:);
         test_set   = data (training (p,k) == 0,:,:);
         test_label = label(training (p,k) == 0,:,:);

        %standardize data
        train_mean  = mean(mean(mean(data)));
        train_set = (train_set - train_mean) ./ 2;
         test_set = ( test_set - train_mean) ./ 2;

        gamma_exp = min_gamma_exp;
            C_exp = min_C_exp;

        %grid search
        while (gamma_exp <= max_gamma_exp);
            gamma = gamma_base ^ gamma_exp;

            while (C_exp <= max_C_exp);
                C = C_base ^ C_exp;

                model = svmtrain (train_label, train_set, sprintf ('-s 0 -t 2 -c %f -g %f', C, gamma));

                [predict_label, accuracy, prob_estimates] = svmpredict (test_label, test_set, model);

                brier_score = getBrierScoreFromSVMResults (test_label, predict_label);

                if (brier_score < best_brier_score);
                   best_gamma = gamma
                   best_C     = C
                   best_brier_score = brier_score
                end;

                C_exp = C_exp + stp_C_exp;
            end;

           gamma_exp = gamma_exp + stp_gamma_exp;
        end;
    end;
 end;

